#pragma once
#include <kr2_program_api/api_v1/bundles/custom_device.h>
#include "communication/interface_provider.h"
#include "communication/bridge/proxy/bridge_proxy.h"
#include "communication/bridge/proxy/bridge_proxy_factory.h"
#include "communication/frontend/frontend_server.h"


#define SOCKET_TIMEOUT 1

#define LOOP_INTERVAL_MS 50

namespace spin {

    class SpinDevice : public kr2_bundle_api::CustomDevice {
    public:
        
        SpinDevice(boost::shared_ptr<kr2_program_api::ProgramInterface> api,
                           const boost::property_tree::ptree &xml_bundle_node);
        
        virtual ~SpinDevice() = default;

        
        //
        //
        // Mandatory event methods
        //
        //
        
        //! Called by the CBun Sandbox process on instantiation (master).
        int onCreate() override;
        //! Called by the CBun Sandbox process on erase (master).
        int onDestroy() override;
        
        //! Called by the User Spawner provess on instantiation (slave).
        int onBind() override;
        //! Called by the User Spawner provess on instantiation (slave).
        int onUnbind() override;
        
        //! System events handling.
        void onHWReady(const kr2_signal::HWReady&);
        void onProgramTerminated(const kr2_signal::ProgramTerminated&);
        
        // Published custom methods, accessible from the CBun interface
        // CBUN_PCALL loadScrew(double screwLength_mm, kr2_program_api::RobotPose &screwPose) const;
        CBUN_PCALL safetyShieldReleaseBit() const; 
        // CBUN_PCALL safetyShieldLockBit() const;
        // CBUN_PCALL safetyShieldCoverBit() const;
        // CBUN_PCALL safetyShieldCoverScrew(const double screwLength) const;
        // CBUN_PCALL safetyShieldDoHoming() const;
        // CBUN_PCALL safetyShieldFollowRobot() const;
        // CBUN_PCALL safetyShieldStop() const;
        // CBUN_PCALL startMotor(const int speed_rpm, const int rampDuration_ms) const;
        // CBUN_PCALL stopMotor() const;

    protected:
        
        CBUN_PCALL onActivate(const boost::property_tree::ptree &param_tree) override;
        CBUN_PCALL onDeactivate() override;
        
        CBUN_PCALL onMount(const boost::property_tree::ptree &param_tree) override;
        CBUN_PCALL onUnmount() override;
        
    private:
        std::shared_ptr<InterfaceProvider> interfaceProvider_;
        std::unique_ptr<FrontendServer> frontendServer_;
        
    };
} // namespace spin
